@@ -28,7 +28,10 @@ class Instrument:
 
     def setup(self, vehicle, **config):
         self.vehicle = vehicle
-        if not self.is_supported:
+        try:
+            if not self.is_supported:
+                return False
+        except:
             return False
 
         self.configurate(**config)
@@ -314,7 +317,7 @@ class Position(Instrument):
 
 class DoorLock(Instrument):
     def __init__(self):
-        super().__init__(component="lock", attr="door_locked", name="Door locked")
+        super().__init__(component="lock", attr="door_locked", name="Door locked", icon="mdi:car-door-lock")
 
     def configurate(self, **config):
         self.spin = config.get('spin', '')
@@ -420,7 +423,6 @@ class RequestHonkAndFlash(Switch):
             'last_timestamp': self.vehicle.honkandflash_action_timestamp
         }
 
-
 class RequestFlash(Switch):
     def __init__(self):
         super().__init__(attr="request_flash", name="Start flashing", icon="mdi:car-parking-lights")
@@ -449,7 +451,6 @@ class RequestFlash(Switch):
             'last_timestamp': self.vehicle.honkandflash_action_timestamp
         }
 
-
 class RequestUpdate(Switch):
     def __init__(self):
         super().__init__(attr="refresh_data", name="Force data refresh", icon="mdi:car-connected")
@@ -481,7 +482,6 @@ class RequestUpdate(Switch):
             'last_timestamp': self.vehicle.refresh_action_timestamp
         }
 
-
 class ElectricClimatisation(Switch):
     def __init__(self):
         super().__init__(attr="electric_climatisation", name="Electric Climatisation", icon="mdi:radiator")
@@ -514,7 +514,6 @@ class ElectricClimatisation(Switch):
             attrs['last_timestamp'] = self.vehicle.climater_action_timestamp
         return attrs
 
-
 class AuxiliaryClimatisation(Switch):
     def __init__(self):
         super().__init__(attr="auxiliary_climatisation", name="Auxiliary Climatisation", icon="mdi:radiator")
@@ -545,7 +544,6 @@ class AuxiliaryClimatisation(Switch):
             'last_timestamp': self.vehicle.climater_action_timestamp
         }
 
-
 class Charging(Switch):
     def __init__(self):
         super().__init__(attr="charging", name="Charging", icon="mdi:battery")
@@ -573,6 +571,67 @@ class Charging(Switch):
             'last_timestamp': self.vehicle.charger_action_timestamp
         }
 
+class MaxChargeCurrent(Switch):
+    def __init__(self):
+        super().__init__(attr="charge_max_ampere", name="Max Charge Current", icon="mdi:battery-charging-outline")
+
+    @property
+    def state(self):
+        # Check charge max ampere valuel. Return True if set to maximum
+        # Return False (Off) for all other values
+        value = self.vehicle.charge_max_ampere
+        if isinstance(value, str):
+            return True if value.lower() == 'maximum' else False
+        elif isinstance(value, int):
+            return True if value == 254 else False
+        return False
+
+    async def turn_on(self):
+        await self.vehicle.set_charger_current('Maximum')
+        await self.vehicle.update()
+
+    async def turn_off(self):
+        await self.vehicle.set_charger_current('Reduced')
+        await self.vehicle.update()
+
+    @property
+    def assumed_state(self):
+        return False
+
+    @property
+    def attributes(self):
+        return {
+            'charge_current': self.vehicle.charge_max_ampere,
+            'last_result': self.vehicle.charger_action_status,
+            'last_timestamp': self.vehicle.charger_action_timestamp
+        }
+
+class PlugAutoUnlock(Switch):
+    def __init__(self):
+        super().__init__(attr="plug_autounlock", name="Plug Auto Unlock", icon="mdi:battery-lock")
+
+    @property
+    def state(self):
+        return self.vehicle.plug_autounlock
+
+    async def turn_on(self):
+        await self.vehicle.set_plug_autounlock('Permanent')
+        await self.vehicle.update()
+
+    async def turn_off(self):
+        await self.vehicle.set_plug_autounlock('Off')
+        await self.vehicle.update()
+
+    @property
+    def assumed_state(self):
+        return False
+
+    @property
+    def attributes(self):
+        return {
+            'last_result': self.vehicle.charger_action_status,
+            'last_timestamp': self.vehicle.charger_action_timestamp
+        }
 
 class WindowHeater(Switch):
     def __init__(self):
@@ -598,9 +657,98 @@ class WindowHeater(Switch):
     def attributes(self):
         return {
             'last_result': self.vehicle.climater_action_status,
-            'last_timestamp': self.vehicle.climater_action_timestamp
+            'last_timestamp': self.vehicle.climater_action_timestamp,
+            "Front":self.vehicle.window_heater_attributes.get("Front", ''),
+            "Rear": self.vehicle.window_heater_attributes.get("Rear", '')
         }
 
+class WindowHeaterNew(Switch):
+    def __init__(self):
+        super().__init__(attr="window_heater_new", name="Window Heater", icon="mdi:car-defrost-rear")
+
+    @property
+    def state(self):
+        return self.vehicle.window_heater_new
+
+    async def turn_on(self):
+        await self.vehicle.set_window_heating('start')
+        await self.vehicle.update()
+
+    async def turn_off(self):
+        await self.vehicle.set_window_heating('stop')
+        await self.vehicle.update()
+
+    @property
+    def assumed_state(self):
+        return False
+
+    @property
+    def attributes(self):
+        return {
+            'last_result': self.vehicle.climater_action_status,
+            'last_timestamp': self.vehicle.climater_action_timestamp,
+            "Front":self.vehicle.window_heater_attributes.get("Front", ''),
+            "Rear": self.vehicle.window_heater_attributes.get("Rear", '')
+        }
+
+class ClimatisationWindowHeat(Switch):
+    def __init__(self):
+        super().__init__(attr="climatisation_window_heat", name="Climatisation Window Heat", icon="mdi:car-defrost-rear")
+
+    @property
+    def state(self):
+        return self.vehicle.climatisation_window_heat
+
+    async def turn_on(self):
+        await self.vehicle.set_window_heating('enabled')
+        await self.vehicle.update()
+
+    async def turn_off(self):
+        await self.vehicle.set_window_heating('disabled')
+        await self.vehicle.update()
+
+    @property
+    def assumed_state(self):
+        return False
+
+    @property
+    def attributes(self):
+        return {
+            'last_result': self.vehicle.climater_action_status,
+            'last_timestamp': self.vehicle.climater_action_timestamp,
+            "Front":self.vehicle.window_heater_attributes.get("Front", ''),
+            "Rear": self.vehicle.window_heater_attributes.get("Rear", '')
+        }
+
+class AuxHeaterDeparture(Switch):
+    def __init__(self):
+        super().__init__(attr="aux_heater_for_departure", name="Allow aux heater next departure", icon="mdi:car-defrost-rear")
+
+    def configurate(self, **config):
+        self.spin = config.get('spin', '')
+
+    @property
+    def state(self):
+        return self.vehicle.aux_heater_for_departure
+
+    async def turn_on(self):
+        await self.vehicle.set_heatersource(source='automatic', spin=self.spin)
+        await self.vehicle.update()
+
+    async def turn_off(self):
+        await self.vehicle.set_heatersource(source='electric')
+        await self.vehicle.update()
+
+    @property
+    def assumed_state(self):
+        return False
+
+    @property
+    def attributes(self):
+        return {
+            'last_result': self.vehicle.timer_action_status,
+            'last_timestamp': self.vehicle.timer_action_timestamp,
+        }
 
 class SeatHeatingFrontLeft(Switch):
     def __init__(self):
@@ -611,14 +759,12 @@ class SeatHeatingFrontLeft(Switch):
         return self.vehicle.seat_heating_front_left
 
     async def turn_on(self):
-        #await self.vehicle.set_seat_heating('start')
-        #await self.vehicle.update()
-        pass
+        await self.vehicle.set_seat_heating('frontLeft', 'enable')
+        await self.vehicle.update()
 
     async def turn_off(self):
-        #await self.vehicle.set_seat_heating('stop')
-        #await self.vehicle.update()
-        pass
+        await self.vehicle.set_seat_heating('frontLeft', 'disable')
+        await self.vehicle.update()
 
     @property
     def assumed_state(self):
@@ -640,14 +786,12 @@ class SeatHeatingFrontRight(Switch):
         return self.vehicle.seat_heating_front_right
 
     async def turn_on(self):
-        #await self.vehicle.set_seat_heating('start')
-        #await self.vehicle.update()
-        pass
+        await self.vehicle.set_seat_heating('frontRight', 'enable')
+        await self.vehicle.update()
 
     async def turn_off(self):
-        #await self.vehicle.set_seat_heating('stop')
-        #await self.vehicle.update()
-        pass
+        await self.vehicle.set_seat_heating('frontRight', 'disable')
+        await self.vehicle.update()
 
     @property
     def assumed_state(self):
@@ -669,14 +813,12 @@ class SeatHeatingRearLeft(Switch):
         return self.vehicle.seat_heating_rear_left
 
     async def turn_on(self):
-        #await self.vehicle.set_seat_heating('start')
-        #await self.vehicle.update()
-        pass
+        await self.vehicle.set_seat_heating('rearLeft', 'enable')
+        await self.vehicle.update()
 
     async def turn_off(self):
-        #await self.vehicle.set_seat_heating('stop')
-        #await self.vehicle.update()
-        pass
+        await self.vehicle.set_seat_heating('rearLeft', 'disable')
+        await self.vehicle.update()
 
     @property
     def assumed_state(self):
@@ -698,14 +840,12 @@ class SeatHeatingRearRight(Switch):
         return self.vehicle.seat_heating_rear_right
 
     async def turn_on(self):
-        #await self.vehicle.set_seat_heating('start')
-        #await self.vehicle.update()
-        pass
+        await self.vehicle.set_seat_heating('rearRight', 'enable')
+        await self.vehicle.update()
 
     async def turn_off(self):
-        #await self.vehicle.set_seat_heating('stop')
-        #await self.vehicle.update()
-        pass
+        await self.vehicle.set_seat_heating('rearRight', 'disable')
+        await self.vehicle.update()
 
     @property
     def assumed_state(self):
@@ -718,7 +858,6 @@ class SeatHeatingRearRight(Switch):
             'last_timestamp': self.vehicle.aircon_action_timestamp
         }
 
-
 class AirConditionAtUnlock(Switch):
     def __init__(self):
         super().__init__(attr="aircon_at_unlock", name="Air-conditioning at unlock", icon="mdi:power-plug")
@@ -728,10 +867,12 @@ class AirConditionAtUnlock(Switch):
         return self.vehicle.aircon_at_unlock
 
     async def turn_on(self):
-        pass
+        await self.vehicle.set_aircon_at_unlock(True)
+        await self.vehicle.update()
 
     async def turn_off(self):
-        pass
+        await self.vehicle.set_aircon_at_unlock(False)
+        await self.vehicle.update()
 
     @property
     def assumed_state(self):
@@ -744,7 +885,6 @@ class AirConditionAtUnlock(Switch):
             'last_timestamp': self.vehicle.aircon_action_timestamp
         }
 
-
 class BatteryClimatisation(Switch):
     def __init__(self):
         super().__init__(attr="climatisation_without_external_power", name="Climatisation from battery", icon="mdi:power-plug")
@@ -772,7 +912,6 @@ class BatteryClimatisation(Switch):
             'last_timestamp': self.vehicle.climater_action_timestamp
         }
 
-
 class PHeaterHeating(Switch):
     def __init__(self):
         super().__init__(attr="pheater_heating", name="Parking Heater Heating", icon="mdi:radiator")
@@ -804,7 +943,6 @@ class PHeaterHeating(Switch):
             'last_timestamp': self.vehicle.pheater_action_timestamp
         }
 
-
 class PHeaterVentilation(Switch):
     def __init__(self):
         super().__init__(attr="pheater_ventilation", name="Parking Heater Ventilation", icon="mdi:radiator")
@@ -870,7 +1008,6 @@ class DepartureTimer1(Switch):
     def attributes(self):
         return dict(self.vehicle.departure1)
 
-
 class DepartureTimer2(Switch):
     def __init__(self):
         super().__init__(attr="departure2", name="Departure timer 2", icon="mdi:radiator")
@@ -1030,12 +1167,16 @@ def create_instruments():
         RequestFlash(),
         RequestHonkAndFlash(),
         RequestUpdate(),
+        PlugAutoUnlock(),
+        AuxHeaterDeparture(),
         WindowHeater(),
-        #SeatHeatingFrontLeft(), # Not yet implemented
-        #SeatHeatingFrontRight(), # Not yet implemented
-        #SeatHeatingRearLeft(), # Not yet implemented
-        #SeatHeatingRearRight(), # Not yet implemented
-        #AirConditionAtUnlock(), # Not yet implemented
+        WindowHeaterNew(),
+        ClimatisationWindowHeat(),
+        SeatHeatingFrontLeft(),
+        SeatHeatingFrontRight(),
+        SeatHeatingRearLeft(),
+        SeatHeatingRearRight(),
+        AirConditionAtUnlock(),
         BatteryClimatisation(),
         ElectricClimatisation(),
         AuxiliaryClimatisation(),
@@ -1045,6 +1186,7 @@ def create_instruments():
         #CombustionClimatisationClimate(),
         CarInfo(),
         Charging(),
+        MaxChargeCurrent(),
         RequestResults(),
         ChargingPower(),
         DepartureTimer1(),
@@ -1222,6 +1364,84 @@ def create_instruments():
             icon="mdi:flash",
             unit="kWh/100 km",
         ),
+        Sensor(
+            attr="trip_last_total_electric_consumption",
+            name="Last trip total electric consumption",
+            icon="mdi:car-battery",
+            unit="kWh/100 km",
+        ),
+        Sensor(
+            attr="trip_last_start_mileage",
+            name="Last trip start mileage",
+            icon="mdi:map-marker-distance",
+            unit="km",
+        ),
+        Sensor(
+            attr="trip_longterm_average_speed",
+            name="Longterm average speed",
+            icon="mdi:speedometer",
+            unit="km/h",
+        ),
+        Sensor(
+            attr="trip_longterm_average_electric_consumption",
+            name="Longterm average electric consumption",
+            icon="mdi:car-battery",
+            unit="kWh/100 km",
+        ),
+        Sensor(
+            attr="trip_longterm_average_fuel_consumption",
+            name="Longterm average fuel consumption",
+            icon="mdi:fuel",
+            unit="l/100 km",
+        ),
+        Sensor(
+            attr="trip_longterm_duration",
+            name="Longterm duration",
+            icon="mdi:clock",
+            unit="min",
+        ),
+        Sensor(
+            attr="trip_longterm_length",
+            name="Longterm distance",
+            icon="mdi:map-marker-distance",
+            unit="km",
+        ),
+        Sensor(
+            attr="trip_longterm_recuperation",
+            name="Longterm recuperation",
+            icon="mdi:battery-plus",
+            unit="kWh/100 km",
+        ),
+        Sensor(
+            attr="trip_longterm_average_recuperation",
+            name="Longterm average recuperation",
+            icon="mdi:battery-plus",
+            unit="kWh/100 km",
+        ),
+        Sensor(
+            attr="trip_longterm_average_auxillary_consumption",
+            name="Longterm average auxillary consumption",
+            icon="mdi:flash",
+            unit="kWh/100 km",
+        ),
+        Sensor(
+            attr="trip_longterm_average_aux_consumer_consumption",
+            name="Longterm average auxillary consumer consumption",
+            icon="mdi:flash",
+            unit="kWh/100 km",
+        ),
+        Sensor(
+            attr="trip_longterm_total_electric_consumption",
+            name="Longterm total electric consumption",
+            icon="mdi:car-battery",
+            unit="kWh/100 km",
+        ),
+        Sensor(
+            attr="trip_longterm_start_mileage",
+            name="Longterm start mileage",
+            icon="mdi:map-marker-distance",
+            unit="km",
+        ),
         Sensor(
             attr="model_image_large",
             name="Model image URL (Large)",
@@ -1232,12 +1452,6 @@ def create_instruments():
             name="Model image URL (Small)",
             icon="mdi:file-image",
         ),
-        Sensor(
-            attr="trip_last_total_electric_consumption",
-            name="Last trip total electric consumption",
-            icon="mdi:car-battery",
-            unit="kWh/100 km",
-        ),
         Sensor(
             attr="pheater_status",
             name="Parking Heater heating/ventilation status",
@@ -1282,7 +1496,8 @@ def create_instruments():
             attr="door_locked",
             name="Doors locked",
             device_class="lock",
-            reverse_state=False
+            reverse_state=False,
+            icon="mdi:car-door-lock"
         ),
         BinarySensor(
             attr="door_closed_left_front",
@@ -1388,31 +1603,31 @@ def create_instruments():
             name="Request in progress",
             device_class="connectivity"
         ),
-        BinarySensor(
-            attr="seat_heating_front_left",
-            name="Seat heating front left",
-            device_class="heat"
-        ),
-        BinarySensor(
-            attr="seat_heating_front_right",
-            name="Seat heating front right",
-            device_class="heat"
-        ),
-        BinarySensor(
-            attr="seat_heating_rear_left",
-            name="Seat heating rear left",
-            device_class="heat"
-        ),
-        BinarySensor(
-            attr="seat_heating_rear_right",
-            name="Seat heating rear right",
-            device_class="heat"
-        ),
-        BinarySensor(
-            attr="aircon_at_unlock",
-            name="Air-conditioning at unlock",
-            device_class=None
-        ),
+        #BinarySensor(
+        #    attr="seat_heating_front_left",
+        #    name="Seat heating front left",
+        #    device_class="heat"
+        #),
+        #BinarySensor(
+        #    attr="seat_heating_front_right",
+        #    name="Seat heating front right",
+        #    device_class="heat"
+        #),
+        #BinarySensor(
+        #    attr="seat_heating_rear_left",
+        #    name="Seat heating rear left",
+        #    device_class="heat"
+        #),
+        #BinarySensor(
+        #    attr="seat_heating_rear_right",
+        #    name="Seat heating rear right",
+        #    device_class="heat"
+        #),
+        #BinarySensor(
+        #    attr="aircon_at_unlock",
+        #    name="Air-conditioning at unlock",
+        #    device_class=None
+        #),
     ]
 
 
@@ -1424,5 +1639,4 @@ class Dashboard:
             for instrument in create_instruments()
             if instrument.setup(vehicle, **config)
         ]
-        _LOGGER.debug("Supported instruments: " + ", ".join(str(inst.attr) for inst in self.instruments))
-
+        _LOGGER.debug("Supported instruments: " + ", ".join(str(inst.attr) for inst in self.instruments))
\ No newline at end of file
